//
//  AppDelegate.swift
//  ZoopCheckout
//
//  Created by Carla Galdino Wanderley on 11/10/17.
//  Copyright © 2017 Zoop. All rights reserved.
//

import UIKit
import EMVConnectiOS


@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate, DeviceProtocol {

    func deviceDidConnect(device: Device) {
        print("Conectado acessorio: \(device.getName())")
        goToTransactionViewController()
    }
    
    func deviceDidDisconnect(device: Device) {
        print("Removido acessorio: \(device.getName())")
        goToDevicesViewController()
    }
    
    func devicesListDidUpdate() {
        print("devicesListDidUpdate")
        NotificationCenter.default.post(name: Notification.Name("ZSDK_DEVICE_LIST_UPDATED"), object: nil)
    }
    
    let deviceManager: DevicesManager = DevicesManager.shared
    
    var window: UIWindow?
    
    override init() {
        
        super.init()
        
        deviceManager.setDeviceListener(deviceListener: self)
    }
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        return true
    }
    
    private func goToTransactionViewController() {
        if let currentViewController = AppDelegate.getCurrentViewController() as? DevicesViewController {
            currentViewController.performSegue(withIdentifier: "PresentTestTransactionSegue", sender: currentViewController)
        }
        
//        let mainStoryboard : UIStoryboard = UIStoryboard(name: "Main", bundle: nil)
//        let viewController : UIViewController = mainStoryboard.instantiateViewController(withIdentifier: "TransactionTestViewController")
//        self.window = UIWindow(frame: UIScreen.main.bounds)
//        self.window?.rootViewController = viewController
//        self.window?.makeKeyAndVisible()
    }

    private func goToDevicesViewController() {
        AppDelegate.getNavigationController()?.popToRootViewController(animated: true)
//        let mainStoryboard : UIStoryboard = UIStoryboard(name: "Main", bundle: nil)
//        let viewController : UIViewController = mainStoryboard.instantiateInitialViewController()!
//
//        self.window = UIWindow(frame: UIScreen.main.bounds)
//        self.window?.rootViewController = viewController
//        self.window?.makeKeyAndVisible()
    }

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }

    // Returns the most recently presented UIViewController (visible)
    class func getCurrentViewController() -> UIViewController? {
        
        // If the root view is a navigation controller, we can just return the visible ViewController
        if let navigationController = getNavigationController() {
            
            return navigationController.visibleViewController
        }
        
        // Otherwise, we must get the root UIViewController and iterate through presented views
        if let rootController = UIApplication.shared.keyWindow?.rootViewController {
            
            var currentController: UIViewController! = rootController
            
            // Each ViewController keeps track of the view it has presented, so we
            // can move from the head to the tail, which will always be the current view
            while( currentController.presentedViewController != nil ) {
                
                currentController = currentController.presentedViewController
            }
            return currentController
        }
        return nil
    }
    
    // Returns the navigation controller if it exists
    class func getNavigationController() -> UINavigationController? {
        
        if let navigationController = UIApplication.shared.keyWindow?.rootViewController  {
            
            return navigationController as? UINavigationController
        }
        return nil
    }
}

