//
//  DevicesViewController.swift
//  ZoopCheckout
//
//  Created by Carla Galdino Wanderley on 11/10/17.
//  Copyright © 2017 Zoop. All rights reserved.
//

import UIKit
import EMVConnectiOS

class DevicesViewController: UITableViewController  {
    
    let deviceManager: DevicesManager = DevicesManager.shared

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
        NotificationCenter.default.addObserver(self, selector: #selector(listShouldUpdate), name: Notification.Name("ZSDK_DEVICE_LIST_UPDATED"), object: nil)
        
    }
    
    @objc public func listShouldUpdate() {
        print("Atualizando dados da tabela")
        self.tableView.reloadData()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        deviceManager.searchDevice()
        NotificationCenter.default.addObserver(self, selector: #selector(listShouldUpdate), name: Notification.Name("ZSDK_DEVICE_LIST_UPDATED"), object: nil)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.tableView.reloadData()
    }    
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
    }
    
    func getAccessoryIdentifiers() -> [String]{
        let name = deviceManager.getConnectedDevicesList().map { $0.getName() }
        print("listando device -> \(name)")
        return name
    }
    
    //MARK: - TableView
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "LabelCell", for: indexPath)
        cell.textLabel?.text = getAccessoryIdentifiers()[indexPath.row]
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return "Dispositivos disponíveis"
    }
    
    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return deviceManager.getConnectedDevicesList().count
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let device = deviceManager.getConnectedDevicesList()[indexPath.row]

        deviceManager.selectDevice(device: device)
    }
}
