//
//  TransactionTestViewController.swift
//  ZoopCheckout
//
//  Created by Carla Galdino Wanderley on 16/10/17.
//  Copyright © 2017 Zoop. All rights reserved.
//

import UIKit
import EMVConnectiOS

class TransactionTestViewController: UIViewController, UITextFieldDelegate, ExtraCardInformationProtocol, TerminalPaymentProtocol, ApplicationDisplayProtocol {

    
    @IBOutlet weak var amountTextField: UITextField!
    @IBOutlet weak var messageLabel: UILabel!
    @IBOutlet weak var commentLabel: UILabel!
    @IBOutlet weak var finalMessage: UILabel!
    @IBOutlet weak var nInstallments: UITextField!
    @IBOutlet weak var voidButton: UIButton!
    @IBOutlet weak var cancelButton: UIButton!
    @IBOutlet weak var signatureButton: UIButton!
    
    var transactionId:String?
    var transactionAmount:String?
    var transactionStatus:String?
    
    var zoopTerminalPayment:ZoopTerminalPayment?
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.voidButton.isEnabled = false
        self.cancelButton.isEnabled = false
        
        self.amountTextField.delegate = self
        self.amountTextField.keyboardType = UIKeyboardType.decimalPad
        
        self.nInstallments.delegate = self
        self.nInstallments.keyboardType = UIKeyboardType.decimalPad
        
        self.addDoneButtonOnKeyboard()
        
        self.zoopTerminalPayment = ZoopTerminalPayment()
        
        self.cancelButton.setTitleColor(.gray, for: .normal)
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
//        EADSessionController.shared().closeSession()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func charge(paymentOption: Int, nInstallments: Int) {
        // 0 for credit | 1 for debit
        if let chargeAmount = amountTextField.text, !chargeAmount.isEmpty
        {
            if let number = Double(chargeAmount.replacingOccurrences(of: ",", with: ".")) {
                // Seller info
                let sMarketplaceId = "3249465a7753536b62545a6a684b0000"
                let sSellerId = "1e5ee2e290d040769806c79e6ef94ee1"
                let sPublishableKey = "zpk_test_EzCkzFFKibGQU6HFq7EYVuxI"
                
                /* Rodrigo
                 let sMarketplaceId = "0162893710a6495e86542eeff192baa1"
                 let sSellerId = "f342a002b03e4964874cbaf095a20274"
                 let sPublishableKey = "zpk_prod_O6Wo7kRzYX11U0KQCy5Yl9hx"
                 */
                
                self.zoopTerminalPayment?.setTerminalPaymentListener(ptpl: self)
                self.zoopTerminalPayment?.setApplicationDisplayListener(padl: self)
                self.zoopTerminalPayment?.setExtraCardInformationListener(pExtraCardInformationListener: self)
                
                var _paymentOption :PaymentType?
                
                if paymentOption == 0 && nInstallments == 1 {
                    _paymentOption = PaymentType.credit
                } else if paymentOption == 0 && nInstallments > 1 {
                    _paymentOption = PaymentType.installment_credit
                } else if paymentOption == 1 {
                    _paymentOption = PaymentType.debit
                }
                
                
                do {
                    try self.zoopTerminalPayment?.charge(valueToCharge: number, paymentOption: _paymentOption!, iNumberOfInstallments: nInstallments, marketplaceId: sMarketplaceId, sellerId: sSellerId, publishableKey: sPublishableKey)
                } catch {
                    print("Erro ao enviar comando charge")
                }

            }
            
        } else {
            let alert = UIAlertController(title: "Valor inválido", message: "Por favor, insira um valor em reais para a transação", preferredStyle: UIAlertController.Style.alert)
            alert.addAction(UIAlertAction(title: "Ok", style: UIAlertAction.Style.default, handler: nil))
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    @IBAction func chargeCredit(_ sender: Any) {
        
        var intnInstallments = 1
        var paymentOption = 0
        
        if let nInstallmentsFromText = nInstallments.text, !nInstallmentsFromText.isEmpty {
            intnInstallments = Int(nInstallmentsFromText)!
            paymentOption = 2
        }
        
        self.charge(paymentOption: paymentOption, nInstallments:intnInstallments)
    }
    
    @IBAction func chargeDebit(_ sender: Any) {
        self.charge(paymentOption: 1, nInstallments: 1)
    }
    
    @IBAction func cancelDidTap(_ sender: Any) {
        print("Operação cancelada.")
        self.zoopTerminalPayment?.requestCancel()
        ZoopSDKErrorHandler.sharedInstance.notifyFailure()
    }
    
    @IBAction func signatureCaptureDidTap(_ sender: Any) {
        print("Not implemented yet")
    }
    
    @IBAction func configTerminalDidTap(_ sender: Any) {
        print("Not implemented yet")
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        return false
    }
    
    // ExtraCardInformationProtocol
    func cardLast4DigitsRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira os últimos 4 dígitos do cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "Últimos 4 dígitos"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }
    
    func cardExpirationDateRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira a data de validade do seu cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "dd/mm/aaaa"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }
    
    func cardCVCRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira o código verificador do cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "CVC"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }
    
    //TerminalPaymentProtocol
    func paymentFailed(var1:AnyObject)
    {
        DispatchQueue.main.async {
            self.finalMessage.text = "Transação falhou."
        }
    }
    
    func paymentSuccessful(var1:AnyObject)
    {
        DispatchQueue.main.async {
            self.finalMessage.text = "Transação realizada com sucesso."
            self.voidButton.isEnabled = true
        }
        print(var1)
        if let obj = var1 as? Dictionary<String, AnyObject> {
            print("Entrou conversao de tipos")
            print(obj.keys)
            print(obj["id"] ?? "id nao definido")
            self.transactionId = obj["id"] as? String
            print(obj["status"] ?? "status nao definido")
            self.transactionStatus = obj["status"] as? String
            print(obj["amount"] ?? "amount nao definido")
            self.transactionAmount = obj["amount"] as? String
        }
    }
    
    func paymentAborted()
    {
        DispatchQueue.main.async {
            self.finalMessage.text = "Transação interrompida."
        }
    }
    
    func cardholderSignatureRequested()
    {
        print("cardholderSignatureRequested")
        DispatchQueue.main.async {
            self.performSegue(withIdentifier: "showSignatureViewSegue", sender: self)
        }
    }
    
    func currentChargeCanBeAbortedByUser(var1:Bool)
    {
        print("currentChargeCanBeAbortedByUser")
        self.cancelButton.setTitleColor(.black, for: .normal)
        self.cancelButton.isEnabled = true
    }
    
    func signatureResult(var1:Int)
    {
        print("signatureResult")
    }
    
    func addDoneButtonOnKeyboard() {
        let doneToolbar: UIToolbar = UIToolbar(frame: CGRect(x: 0, y: 0, width: 320, height: 50))
        doneToolbar.barStyle       = UIBarStyle.default
        let flexSpace              = UIBarButtonItem(barButtonSystemItem: UIBarButtonItem.SystemItem.flexibleSpace, target: nil, action: nil)
        let done: UIBarButtonItem  = UIBarButtonItem(title: "Done", style: UIBarButtonItem.Style.done, target: self, action: #selector(self.doneButtonAction))
        
        var items = [UIBarButtonItem]()
        items.append(flexSpace)
        items.append(done)
        
        doneToolbar.items = items
        doneToolbar.sizeToFit()
        
        self.amountTextField.inputAccessoryView = doneToolbar
        self.nInstallments.inputAccessoryView = doneToolbar
    }
    
    @objc func doneButtonAction() {
        self.amountTextField.resignFirstResponder()
        self.nInstallments.resignFirstResponder()
    }
    
    //ApplicationDisplayProtocol
    func showMessage(var1:String, var2:TerminalMessageType)
    {
        DispatchQueue.main.async {
            self.messageLabel.text = var1
        }
    }
    
    func showMessage(var1:String, var2:TerminalMessageType, var3:String)
    {
        DispatchQueue.main.async {
            self.messageLabel.text =  var1
            self.commentLabel.text = var3
        }
    }
    
    func pixPaymentSuccessful(var1: AnyObject) {
        print("Not implemented yet")
    }
    
    func pixPaymentFailed(var1: AnyObject) {
        print("Not implemented yet")
    }
    
    func paymentDuplicated(var1: AnyObject) {
        DispatchQueue.main.async {
            self.finalMessage.text = "Transação Duplicada."
        }
    }
    
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "VoidTransactionSegue" {
            let nextScene =  segue.destination as! VoidTransactionTestViewController
            
            nextScene.setTransactionId(id: self.transactionId!)
            nextScene.setTransactionStatus(status: self.transactionStatus!)
            nextScene.setTransactionAmount(amount: self.transactionAmount!)
        }
    }
}
