//
//  VoidTransactionTestViewController2.swift
//  ZoopDemoAppiOS
//
//  Created by Carla Galdino Wanderley on 07/11/17.
//  Copyright © 2017 Zoop. All rights reserved.
//

import UIKit
import EMVConnectiOS

class VoidTransactionTestViewController: UIViewController, VoidTransactionProtocol, ApplicationDisplayProtocol, ExtraCardInformationProtocol {
    
    var zoopTerminalVoidPayment: ZoopTerminalVoidPayment?
    
    var transactionId:String?
    var transactionStatus:String?
    var transactionAmount:String?
    
    @IBOutlet weak var terminalCommentsLabel: UILabel!
    @IBOutlet weak var terminalMessageLabel: UILabel!
    @IBOutlet weak var cancelButton: UIButton!
    
    @IBOutlet weak var transactionIdLabel: UILabel!
    @IBOutlet weak var transactionStatusLabel: UILabel!
    @IBOutlet weak var transactionValueLabel: UILabel!
    
    func setTransactionId(id:String) {
        self.transactionId = id
    }
    
    func setTransactionStatus(status:String) {
        self.transactionStatus = status
    }
    
    func setTransactionAmount(amount:String) {
        self.transactionAmount = amount
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.cancelButton.isEnabled = false
        
        self.transactionIdLabel.text = "Id da transação: \(self.transactionId!)"
        self.transactionStatusLabel.text = "Status da transação: \(self.transactionStatus!)"
        self.transactionValueLabel.text = "Valor da transação: \(self.transactionAmount!)"

        // Do any additional setup after loading the view.
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func performVoidTransaction(_ sender: Any) {
        let sMarketplaceId = "3249465a7753536b62545a6a684b0000"
        let sSellerId = "1e5ee2e290d040769806c79e6ef94ee1"
        let sPublishableKey = "zpk_test_EzCkzFFKibGQU6HFq7EYVuxI"
        
        self.zoopTerminalVoidPayment = ZoopTerminalVoidPayment()
        self.zoopTerminalVoidPayment?.setTerminalPaymentListener(ptpl: self)
        self.zoopTerminalVoidPayment?.setApplicationDisplayListener(padl: self)
        self.zoopTerminalVoidPayment?.setExtraCardInformationListener(pExtraCardInformationListener: self)
        
        do {
            try self.zoopTerminalVoidPayment?.voidTransaction(transactionId:self.transactionId!, marketplaceId: sMarketplaceId, sellerId: sSellerId, publishableKey: sPublishableKey)
        } catch {
            print("Erro ao enviar comando Void")
        }
    }
    
    //ApplicationDisplayProtocol
    func showMessage(var1:String, var2:TerminalMessageType)
    {
        DispatchQueue.main.async {
            self.terminalMessageLabel.text = var1
        }
    }
    
    func showMessage(var1:String, var2:TerminalMessageType, var3:String)
    {
        DispatchQueue.main.async {
            self.terminalMessageLabel.text =  var1
            self.terminalCommentsLabel.text = var3
        }
    }
    
    //VoidTransactionProtocol
    func voidTransactionSuccessful(var1:AnyObject) {
        self.terminalMessageLabel.text = "Estorno realizado com sucesso."
    }
    
    func voidTransactionFailed(var1:AnyObject) {
        self.terminalMessageLabel.text = "Erro ao realizar estorno."
    }
    
    func currentVoidTransactionCanBeAbortedByUser(var1:Bool) {
        self.cancelButton.isEnabled = true
    }
    
    // ExtraCardInformationProtocol
    func cardLast4DigitsRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira os últimos 4 dígitos do cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "Últimos 4 dígitos"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }
    
    func cardExpirationDateRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira a data de validade do seu cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "dd/mm/aaaa"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }
    
    func cardCVCRequested()
    {
        //1. Create the alert controller.
        let alert = UIAlertController(title: "Informações adicionais", message: "Por favor, insira o código verificador do cartão.", preferredStyle: .alert)
        
        //2. Add the text field. You can configure it however you need.
        alert.addTextField { (textField) in
            textField.placeholder = "CVC"
        }
        
        // 3. Grab the value from the text field, and print it when the user clicks OK.
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: { [weak alert] (_) in
            let textField = alert?.textFields![0] // Force unwrapping because we know it exists.
            print("Text field: \(String(describing: textField?.text))")
        }))
        
        // 4. Present the alert.
        self.present(alert, animated: true, completion: nil)
    }

    
    @IBAction func cancelButtonDidTap(_ sender: Any) {
        print("Cancelamento estorno ainda não foi implementado.")
    }
    
    func voidPixTransactionList(var1: AnyObject) {
        print("Not implemented yet")
    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
